#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h> 
#include <stdio.h>
#include "slm.h"

#define TEST "test.slm"
#define IMAGE	"test_image/long_bridge.slm"
#define SLM "/dev/slm"
#define TIME	0x00000fff
#define 	QUIT		0
#define 	POWER_ON	1
#define 	POWER_OFF	2
#define 	ENABLE_WRITE	3
#define 	DISABLE_WRITE	4
#define 	UPLOAD_FRAME	5
#define 	RUN		6
#define		DISPLAY		7
#define		PERIOD		8
#define 	HELP		10

void h_err(int);
#define ROW_SIZE (512)
#define IMAGE_LENGTH 1560
#define SIZE 512*IMAGE_LENGTH
#define N 100000
main(){
int* dummy=NULL;
FILE	*fi,*fo;
int	dev;
unsigned char image[SIZE*2];
unsigned char *invimage;
int pos,y,i,option,offset,frame,period;
int n=0,err;
char temp[1024];


	invimage=image+SIZE;
	printf("Matt's test slm driver program\n");
	printf("Opening test image \"%s\"\n",IMAGE);
	fi=fopen(IMAGE,"r");
	if(fi==NULL){
		printf("Image %s not found\n",IMAGE);
		printf("Exiting\n");
		}
	printf("Reading data...");
	fread(image,sizeof(unsigned char),SIZE,fi);
	fclose(fi);
	printf("done\n");
	fo=fopen(TEST,"w");
	for(i=0;i<SIZE;i++){invimage[i]=255-image[i];}

	printf("Opening slm \"%s\"\n",SLM);
	dev=open(SLM,O_RDWR);
	if(dev<0){
		printf("Failed to open %s\n",SLM);
		perror("slm_test:FTO:");
		exit(1);
		}
	printf("periods to %x\n",TIME);
	h_err(ioctl(dev,BNS_SET_PERIODS,TIME));
	do{
		printf("Choose an option (help=%d):\n",HELP);
		scanf("%d",&option);
		switch(option){
			case HELP:
				printf("Power on	=%d\n",POWER_ON);
				printf("Power off	=%d\n",POWER_OFF);
				printf("Enable write	=%d\n",ENABLE_WRITE);
				printf("Disable write /\n Allow image	=%d\n",DISABLE_WRITE);
				printf("Upload frames	=%d\n",UPLOAD_FRAME);
				printf("Run		=%d\n",RUN);
				printf("Display 1 frame	=%d\n",DISPLAY);
				printf("Change period	=%d\n",PERIOD);
				printf("Quit		=%d\n",QUIT);
				break;
			case POWER_ON:
				printf("Setting slm power on\n");
				h_err(ioctl(dev,BNS_POWER_ON,dummy));
				break;
			case POWER_OFF:
				printf("Setting slm power off\n");
				h_err(ioctl(dev,BNS_POWER_OFF,dummy));
				break;
			case ENABLE_WRITE:
				printf("Enabling write\n");
				h_err(ioctl(dev,BNS_ENABLE_WRITE,dummy));
				break;
			case DISABLE_WRITE:
				printf("Disabling write\n");
				h_err(ioctl(dev,BNS_DISABLE_WRITE,dummy));
				break;
			case UPLOAD_FRAME:
				for(n=0,offset=0;n<1024;n++,offset+=2){
					printf("uploading real image to %d \n",n);
					h_err(ioctl(dev,BNS_UPLOAD_IMAGE,n));
					err=ROW_SIZE;
					for(pos=offset,y=0;y<512;y++,pos+=IMAGE_LENGTH){ 
						err=write(dev,image+pos,ROW_SIZE);
						//fwrite(image+pos,sizeof(unsigned char),ROW_SIZE,fo);
						if(err!=ROW_SIZE){
							printf("WRITE ERROR (writes enabled?)\n",err);
							break;
							}
						}
					}

				printf("Done\n");
				break;

			case RUN:
				for(n=0;n<1014;n++){
					printf("%d ",n);
					h_err(ioctl(dev,BNS_DISPLAY_IMAGE,n));
					//scanf("%s",temp);
					usleep(100);
				}
				break;
			case DISPLAY:
				printf("Enter a frame to display: ");
				scanf("%d",&frame);
				h_err(ioctl(dev,BNS_DISPLAY_IMAGE,frame));
				break;
			case PERIOD:
				printf("Enter period length: ");
				scanf("%x",&period);
				printf("\nSetting time to %f seconds (%d)\n",(164.4e-6*(float)period),period);
				h_err(ioctl(dev,BNS_SET_PERIODS,period));
				break;	
                        }
				
		}	
	while(option!=QUIT);
	fclose(fo);
	printf("Setting slm power off\n");
	h_err(ioctl(dev,BNS_POWER_OFF,n));
	printf("Done, exiting\n");
	}	
	
	
void h_err(int err){
	if(err!=0){
		printf("Got error %d\n",err);
		exit(1);
		}
	else{
		printf("Ok\n");
		}
	usleep(1000);
	}
